<?php

namespace payment;

use think\Db;

/**
 * 虎皮椒微信支付
 */
class XunhupayWxPay {
    //插件配置参数
    private $config;

    public function __construct($params = []) {
        $this->config = $params;
    }

    /**
     * 配置信息
     */
    public function Config() {
        // 基础信息
        $base = [
            'name' => '迅虎微信支付',
            'version' => '1.0.0',
            'apply_version' => '不限',
            'apply_terminal' => ['pc','h5','weixin'],// 适用终端 默认全部 ['pc', 'h5', 'app', 'alipay', 'weixin', 'baidu']
            'desc' => '个人微信支付接口，无需提现，100%资金安全，官方清算，金额无限制，支持各种网站系统。是个人收款的最佳解决方案。<a href="https://admin.xunhuweb.com" target="_blank">立即申请</a>',  // 插件描述（支持html）
            'author' => '迅虎网络',  // 开发者
            'author_url' => 'https://www.xunhuweb.com/',  // 开发者主页
        ];

        // 配置信息
        $element = [
            [
                'element' => 'input',
                'type' => 'text',
                'default' => '',
                'name' => 'mchid',
                'placeholder' => 'Mchid',
                'title' => 'Mchid',
                'is_required' => 0,
                'message' => '请填写分配的Mchid',
            ],
            [
                'element' => 'input',
                'type' => 'text',
                'default' => '',
                'name' => 'private_key',
                'placeholder' => 'Private Key',
                'title' => 'Private Key',
                'is_required' => 0,
                'message' => '请填写分配的Private Key',
            ],
            [
                'element' => 'input',
                'type' => 'text',
                'default' => 'https://admin.xunhuweb.com/pay/payment',
                'name' => 'gateway_url',
                'placeholder' => '网关地址',
                'title' => '网关地址',
                'is_required' => 0,
                'message' => '请填写虎皮椒支付网关地址',
            ]
        ];

        return [
            'base' => $base,
            'element' => $element,
        ];
    }


    /**
     * 支付入口
     */
    public function Pay($params = []) {
        // 参数
        if (empty($params)) {
            return DataReturn('参数不能为空', -1);
        }

        // 配置信息
        if (empty($this->config)) {
            return DataReturn('支付缺少配置', -1);
        }
        $requestParams=$this->GetPayParams($params);
      	if($this->is_wechat_app()){
        	$pay_url=$this->data_link('https://admin.xunhuweb.com/pay/cashier',$requestParams);
          	$pay_url1=htmlspecialchars_decode($pay_url,ENT_NOQUOTES);
         	return DataReturn('success', 0,$pay_url1);
        }
		if($this->is_app_client()){
			$redirect_url=$params['redirect_url'];
			$response=$this->http_post_json($this->config['gateway_url'],json_encode($requestParams));
			$result=json_decode($response,true);
			if(!$result||$result['return_code']!='SUCCESS'){
				return DataReturn('fail', -1);
			}
			$sign= $this->generate_xh_hash($result,$this->config['private_key']);
			if(!isset( $result['sign'])|| $sign!=$result['sign']){
				return DataReturn($result['err_msg'], -1);
			}
			$pay_url =$result['mweb_url'];
			$pay_url1=$pay_url.'&redirect_url='.urlencode($redirect_url);
			return DataReturn('success',0,$pay_url1);
		}
        $response=$this->http_post_json($this->config['gateway_url'],json_encode($requestParams));
        $result=json_decode($response,true);
        if(!$result||$result['return_code']!='SUCCESS'){
            return DataReturn('fail', -1);
        }
      	$sign= $this->generate_xh_hash($result,$this->config['private_key']);
        if(!isset( $result['sign'])|| $sign!=$result['sign']){
       		return DataReturn($result['err_msg'], -1);
        }
      $pay_params = [
        'url'       => urlencode(base64_encode($result['code_url'])),
        'order_no'  => $params['order_no'],
        'name'      => urlencode('微信支付'),
        'msg'       => urlencode('打开微信APP扫一扫进行支付'),
        'ajax_url'  => urlencode(base64_encode($params['ajax_url'])),
      ];
      $url = MyUrl('index/pay/qrcode', $pay_params);
      return DataReturn('success', 0, $url);
    }

    /**
     * 支付回调处理
     */
    public function Respond($params = []) {
        //支付异步回调
      $request=(array)json_decode(file_get_contents('php://input'));
      if(isset($request)&&$request['return_code']=='SUCCESS'&&$request['status']=='complete'){
         return DataReturn('支付成功', 0, $this->ReturnData($request));
      }else{
      	return DataReturn('处理异常错误', -100);
      }
    }

    /**
     * 退款处理
     */
    public function Refund($params = []){
        return DataReturn('该支付方式暂未提供退款接口', -1);
    }

    /**
     * 返回数据统一格式
     */
    private function ReturnData($data) {
        $out_trade_no = $data['out_trade_no'];
        $arr = explode('@',$out_trade_no);
        $out_trade_no = $arr['0'];
        // 返回数据固定基础参数
        $data['trade_no']       = $data['transaction_id'];  // 支付平台 - 订单号
        $data['buyer_user']     = $data['mchid'];           // 支付平台 - 用户
        $data['out_trade_no']   = $out_trade_no;  // 本系统发起支付的 - 订单号
        $data['subject']        = $out_trade_no;         // 本系统发起支付的 - 商品名称
        $data['pay_price']      = $data['total_fee']/100;       // 本系统发起支付的 - 总价
        return $data;
    }

    /**
     * 查询
     */

    /**
     * 获取支付参数
     */
    private function GetPayParams($params = []){
        $http_type = ((isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') || (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https')) ? 'https://' : 'http://';
        $data = [
          	'mchid'     	=> $this->config['mchid'],
            'out_trade_no'	=> $params['order_no'].'@'.time(),
            'type'  		=> 'wechat',
            'total_fee' 	=> round($params['total_price'],2)*100,
            'body'  		=> $params['name'].'-订单号:'.$params['order_no'],
            'notify_url'	=>  $params['notify_url'],
            'nonce_str' 	=> str_shuffle(time())
        ];
      	if($this->is_wechat_app()){
          	$data['redirect_url']=$params['redirect_url'];
            $data['sign']=$this->generate_xh_hash($data,$this->config['private_key']);
          	return $data;
        }elseif($this->is_app_client()){
			$data['trade_type'] = 'WAP';
			$data['wap_url']    = $http_type.$_SERVER['SERVER_NAME'];
			$data['wap_name']   = $http_type.$_SERVER['SERVER_NAME'];
			$data['sign']=$this->generate_xh_hash($data,$this->config['private_key']);
          	return $data;
		}else{
         	$data['sign']=$this->generate_xh_hash($data,$this->config['private_key']);
       	 	return $data;
        }
    }

    /**
     * 生成sign
     */
    public static function generate_xh_hash(array $datas,$hashkey){
        ksort($datas);
        reset($datas);

        $pre =array();
        foreach ($datas as $key => $data){
            if(is_null($data)||$data===''){
                continue;
            }
            if($key=='sign'){
                continue;
            }
            $pre[$key]=$data;
        }

        $arg  = '';
        $qty = count($pre);
        $index=0;

        foreach ($pre as $key=>$val){
            $arg.="$key=$val";
            if($index++<($qty-1)){
                $arg.="&";
            }
        }
        return strtoupper(md5($arg.'&key='.$hashkey));
    }
	/**
     * url拼接
     * @param array $url
     * @param string $datas
     */
	public static function data_link($url,$datas){
		ksort($datas);
        reset($datas);
        $pre =array();
        foreach ($datas as $key => $data){
            if(is_null($data)||$data===''){
                continue;
            }
            $pre[$key]=$data;
        }

        $arg  = '';
        $qty = count($pre);
        $index=0;
		 foreach ($pre as $key=>$val){
		 		$val=urlencode($val);
			 	$arg.="$key=$val";
	            if($index++<($qty-1)){
	                $arg.="&amp;";
	            }	
        }
        return $url.'?'.$arg;
	}
    /**
     * http请求
     */
     public static function http_post_json($url, $jsonStr){
	    $ch = curl_init();
	    curl_setopt($ch, CURLOPT_POST, 1);
	    curl_setopt($ch, CURLOPT_URL, $url);
	    curl_setopt($ch, CURLOPT_POSTFIELDS, $jsonStr);
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	    curl_setopt($ch, CURLOPT_HTTPHEADER, array(
	            'Content-Type: application/json; charset=utf-8',
	            'Content-Length: ' . strlen($jsonStr)
	        )
	    );
	    $response = curl_exec($ch);
	    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
	    curl_close($ch);
	 
	    return $response;
	}
    public static function is_wechat_app(){
        return strripos($_SERVER['HTTP_USER_AGENT'],'micromessenger');
    }
	public static function is_app_client(){
        if(!isset($_SERVER['HTTP_USER_AGENT'])){
            return false;
        }
    
        $u=strtolower($_SERVER['HTTP_USER_AGENT']);
        if($u==null||strlen($u)==0){
            return false;
        }
    
        preg_match('/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows ce|xda|xiino/',$u,$res);
    
        if($res&&count($res)>0){
            return true;
        }
    
        if(strlen($u)<4){
            return false;
        }
    
        preg_match('/1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/',substr($u,0,4),$res);
        if($res&&count($res)>0){
            return true;
        }
    
        $ipadchar = "/(ipad|ipad2)/i";
        preg_match($ipadchar,$u,$res);
        if($res&&count($res)>0){
            return true;
        }
    
        return false;
    }
}

?>